##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import Globals, string, zLOG
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.MyMediaManager.Medianet import registerPlugin, configQuery
from Products.MyMediaManager.ServerPlugin import ServerPlugin
from Products.MyMediaManager.interfaces.IOnDemandServerPlugin import IOnDemandServerPlugin

class RealOndemandServer(ServerPlugin):
    """
    Real On-demand Server class
    """
    meta_type = 'Real Media On-demand Server'
    type = 'Real Ondemand Server'

    __implements__ = (IOnDemandServerPlugin
                    , ServerPlugin.__implements__
                    )

    plugin_type = 'Media Server'
    skinname = 'real_ondemand'
    skinpath = '%s/skins' % Globals.package_home(globals())
    security = ClassSecurityInfo()
    _editor_player_skin = 'real_ondemand_editor_player'
    _cinema_player_skin = 'real_ondemand_cinema_player'
    _meta_generator = None
    _protocol = None
    _host = None
    _port = None
    _contentDir = None
    _extension = None
    _upload_login = None
    _upload_password = None
    _upload_dir = None
    _def_cinemaskin = None
    _def_syndicationskin = None
    _width = None
    _height = None
    _provides = ['audio/vnd.rn-realaudio','application/vnd.rn-realmedia']
    _sel_format = None

    factory_type_information = ({'id': 'Real Ondemand Server',
                                             'meta_type': 'Real Media On-demand Server',
                                             'description': 'Use this server to stream on-demand Real Media content',
                                             'product': 'MyMediaManager',
                                             'icon': 'real_icon.gif',
                                             'factory': 'addRealOndemandServer',
                                             'filter_content_types': 0,
                                             'immediate_view': 'real_ondemand_edit_form',
                                             'actions':({'id': 'view',
                                                             'name': 'View',
                                                             'action': 'real_ondemand_view',
                                                             'permissions': ('View',),
                                                             'category': 'object'},
                                                            {'id': 'edit',
                                                             'name': 'Edit',
                                                             'action': 'real_ondemand_edit_form',
                                                             'permissions': ('Manage properties',),
                                                             'category': 'object'}
                                                           )},)

    def __init__(self, id, title=''):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, REQUEST=None):
        """
        Edit the real on-demand server
        """
        req = REQUEST
        if not req:
            req = self.REQUEST
        if req:
            self.setTitle(req.get('title', None))
            self.setDescription(req.get('description', None))

            metagen = req.get('meta_generator', None)
            if metagen is not None:
                self._meta_generator = metagen

            protocol = req.get('protocol', None)
            if protocol is not None:
                self._protocol = protocol

            host = req.get('host', None)
            if host is not None:
                self._host = host

            port = req.get('port', None)
            if port is not None:
                port = string.replace(port, ':', '')
                if len(port) > 0:
                    try:
                        port = int(port)
                    except:
                        raise 'AttributeError', 'Port must be an integer'
                    self._port = port

            dir = req.get('contentDir', None)
            if dir is not None:
                if len(dir) > 0:
                    if not dir[0] == '/':
                        dir = '/%s' % dir
                    if not dir[-1] == '/':
                        dir = '%s/' % dir
                    space_count = string.count(dir, ' ')
                    if space_count > 0:
                        raise 'AttributeError', 'Spaces not allowed in paths'
                else:
                    dir = ''
                self._contentDir = dir

            ext = req.get('extension', None)
            if ext is not None:
                self._extension = ext

            login = req.get('upload_login', None)
            if login is not None:
                self._upload_login = login

            password = req.get('upload_password', None)
            if password is not None:
                if password != 'mmm_password':
                    self._upload_password = password

            dir = req.get('upload_dir', None)
            if dir is not None:
                self._upload_dir = dir

            cinskin = req.get('cinemaskin', None)
            if cinskin is not None:
                self._def_cinemaskin = cinskin

            synskin = req.get('syndicationskin', None)
            if synskin is not None:
                self._def_syndicationskin = synskin

            width = req.get('width', None)
            if width is not None:
                self._width = width

            height = req.get('height', None)
            if height is not None:
                self._height = height
                
            sel_format = req.get('selected_format', None)
            if sel_format is not None:
                self._sel_format = sel_format

            self.reindexObject()
            msg = 'Real Server saved'
            editform = self.getEditMethod()
            req.RESPONSE.redirect('%s?%sportal_status_message=%s' % (editform, configQuery(self), string.replace(msg, ' ', '+')))

    security.declarePublic('getDataListForServerInfo')
    def getDataListForServerInfo(self):
        """
        Return a dictionary containing the info a ServerInfo need as default settings
        """
        width = {'id': 'width', 'title': 'Width', 'value': self.getWidth(), 'type': 'text'}
        height = {'id': 'height', 'title': 'Height', 'value': self.getHeight(), 'type': 'text'}
        skintemplate = getattr(self, 'mediaevent_view_macros')
        #zLOG.LOG('Real Ondemand Server', zLOG.INFO, 'skin template: %s' % str(skintemplate))
        skins = skintemplate.macros.keys()
        skinlist = []
        for skin in skins:
            skindict = {'value': skin, 'content': skin}
            skinlist.append(skindict)
        cinemaskin = {'id': 'cinemaskin', 'title': 'Cinema Skin', 'value': self.getCinemaSkin(), 'type': 'select', 'options': skinlist}
        syndicationskin = {'id': 'syndicationskin', 'title': 'Syndication Skin', 'value': self.getSyndicationSkin(), 'type': 'select', 'options': ''}

        attrlist = [width, height, cinemaskin, syndicationskin]
        if self.getMetaGenerator():
            catalog = getToolByName(self, 'portal_catalog')
            brains = catalog(meta_type=self.getMetaGenerator())
            metaoptions = []
            for brain in brains:
                metaobj = brain.getObject()
                metadict = {'value': metaobj.absolute_url(relative=1), 'content': metaobj.title_or_id()}
                metaoptions.append(metadict)
            metagenerator = {'id': 'meta_generator', 'title': 'Meta Generator', 'value': None, 'type': 'select', 'options': metaoptions}
            attrlist.append(metagenerator)

        return attrlist

    security.declarePublic('getPort')
    def getPort(self):
        """
        Return the port for the server
        """
        return self._port

    security.declarePublic('getContentDir')
    def getContentDir(self):
        """
        Return the content dir for the server
        """
        return self._contentDir

    security.declareProtected(CMFCorePermissions.View, 'getPlayUrl')
    def getPlayUrl(self, filename, out_ext=0):
        """
        Return a url for the requested filename
        """
        if out_ext:
            url = '%s://%s:%d%s%s' % (self.getProtocol(), self.getHost(), self.getPort(), self.getContentDir(), filename)
        else:
            url = '%s://%s:%d%s%s%s' % (self.getProtocol(), self.getHost(), self.getPort(), self.getContentDir(), filename, self.getExtension())
        return url

def addRealOndemandServer(self, id, REQUEST=None):
        portalroot = getToolByName(self, 'portal_url').getPortalObject()
        hasnet = 0
        if 'portal_medianet' in portalroot.objectIds():
            medianet = portalroot.restrictedTraverse('portal_medianet')
            if medianet.meta_type == 'Media Manager Medianet':
                hasnet = 1
        if not hasnet:
            raise "You must have a Media Manager Medianet named 'portal_medianet' in the portal root!!!"
        ob=RealOndemandServer(id)
        self._setObject(id, ob)
        server = getattr(medianet, id)
        server.edit(REQUEST)

Globals.InitializeClass(RealOndemandServer)
registerPlugin(RealOndemandServer)
