##############################################################################
#    Copyright (C) 2002-03  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import zLOG, string, os, Globals
from utils import getUniqueId
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.PageTemplates import PageTemplateFile

_zptdir = os.path.join(Globals.package_home(globals()), 'www')

class ServerInfo(SimpleItem):
    """
    A class that binds a server object to a masterfile
    """

    meta_type = 'Media Manager ServerInfo'
    security = ClassSecurityInfo()
    _width = None
    _height = None

    manage_options=({'label':'Server Info', 'action':'manage_overview',},
                   ) + SimpleItem.manage_options

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile.PageTemplateFile( 'displayServerInfo', _zptdir)

    def __init__(self, masterfile, serverid):
        """
        Make a new instance of a ServerInfo object
        """
        self.id = getUniqueId()
        self._masterfile = masterfile
        self._content_type = None
        self._serverid = serverid
        self._encoder_profile = None
        self._cinemaskin = None
        self._syndicationskin = None
        self._encoder_start = None
        self._width = None
        self._height = None

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, REQUEST=None, datadict=None):
        """
        Edit the ServerInfo object
        """
        if REQUEST:
            req = REQUEST.form
            req_url = REQUEST.get('HTTP_REFERER', None)
            has_query = string.find(req_url, '?')
            if has_query != -1:
                req_url, query = string.split(req_url, '?')
        elif datadict:
            req = datadict
            req_url = None
        else:
            raise 'AttributeError', 'Missing data that should be used to update the serverinformation'

        for key in req.keys():
            if key == 'serverid':
                self._serverid = req[key]
            elif key == 'encoder_profile':
                self._encoder_profile = req[key]
            elif key == 'cinemaskin':
                self._cinemaskin = req[key]
            elif key == 'syndicationskin':
                self._syndicationskin = req[key]
            elif key == 'height':
                self._height = req[key]
            elif key == 'width':
                self._width = req[key]
            elif key == 'content_type':
                self._content_type == req[key]
            elif key == 'encoder_start':
                if req[key] is not None:
                    mnet = getToolByName(self, 'portal_medianet')
                    event = mnet.getEventFor(self)
                    eventdate = event.start().strftime('%Y/%m/%d')
                    if req[key] == '':
                        encstart = event.start()
                    else:
                        encstart = "%s %s" % (str(eventdate), req[key])
                        encstart = DateTime(encstart)
                    self.setEncoderStart(encstart)
            elif key == 'edit':
                continue
            elif key == 'mediafile':
                filedata = req[key]
                if filedata:
                    self.uploadMasterfileData(filedata)
            elif key == 'filename':
                masterfile = self.getMasterfile()
                masterfile.edit(filename=req[key])
            else:
                setattr(self, '_%s' % key, req[key])

        msg = 'Server Information updated'
        if req_url:
            REQUEST.RESPONSE.redirect('%s?serverid=%s&portal_status_message=%s' % (req_url, self.getMediaServer().getId(), string.replace(msg, ' ', '+')))

    security.declarePublic('getValueFor')
    def getValueFor(self, attr, info_only=0):
        """
        Return the value for a specific attribute
        """
        zLOG.LOG(self.meta_type, zLOG.INFO, "getValueFor %s - dict: \n%s" % (attr, str(self.__dict__)))
        value = getattr(self, '_%s' % attr, None)
        if value and callable(value):
            value = value()
        if not value and not info_only:
            #zLOG.LOG(self.meta_type, zLOG.INFO, "getValueFor - getting value from media server")
            server = self.getMediaServer()
            value = getattr(server, '_%s' % attr, None)
            if value and callable(value):
                value = value()
        if value == 'None':
            value = None
        #zLOG.LOG(self.meta_type, zLOG.INFO, "getValueFor: %s == %s" % (attr, value))
        return value

    security.declarePublic('getContentType')
    def getContentType(self):
        """
        Return the content type of the content
        """
        if hasattr(self, '_content_type'):
            return self._content_type
        else:
            return None

    security.declarePublic('getMasterfile')
    def getMasterfile(self):
        """
        Return the masterfile object for this serverinfo object
        """
        return self._masterfile

    security.declarePublic('uploadMasterfileData')
    def uploadMasterfileData(self, data):
        """
        Upload data to a masterfile
        """
        masterfile = self.getMasterfile()
        msg = masterfile.upload(data, self)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setEncoderStart')
    def setEncoderStart(self, encstart=None):
        """
        Set the encoder start time
        """
        if encstart is None:
            encstart = DateTime()
        self._encoder_start = encstart

    security.declarePublic('getEncoderStart')
    def getEncoderStart(self):
        """
        Return the encoder start time
        """
        return self._encoder_start

    security.declarePublic('getMediaServer')
    def getMediaServer(self):
        """
        Return the media server corresponding to the serverid
        """
        medianet = getToolByName(self, 'portal_medianet')
        return medianet.getServerById(self._serverid)

    security.declarePublic('getInfoAttrs')
    def getInfoAttrs(self):
        server = self.getMediaServer()
        serverattrs = server.getDataListForServerInfo()

        for attrdict in serverattrs:
            id = attrdict['id']
            infoattr = getattr(self, '_%s' % id, None)
            if infoattr:
                attrdict['value']= infoattr
        attrlist = serverattrs
        return attrlist
