##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import Globals, string, zLOG
from AccessControl import ClassSecurityInfo
from Products.CMFCore.Expression import Expression
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.MyMediaManager.PluginBase import PluginBase
from Products.MyMediaManager.utils import getUniqueId
from Products.MyMediaManager.Medianet import registerPlugin
from Products.MyMediaManager.interfaces.IServerPlugin import IServerPlugin

class ServerPlugin(PluginBase):
    """
    Base Server Plugin class
    """
    manage_options = ( ( { 'label'  : 'Configuration'
                         , 'action' : 'manage_Configuration'
                         }
                        ,
                       )
                     + PluginBase.manage_options
                     )

    __implements__ = (IServerPlugin
                    , PluginBase.__implements__
                    )

    security = ClassSecurityInfo()

    def __init__(self, id, title=''):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.setTitle(title)

    security.declarePublic('getProtocol')
    def getProtocol(self):
        """
        Return the protocol for the server
        """
        return self._protocol

    security.declarePublic('getHost')
    def getHost(self):
        """
        Return the hostname or IP for the server
        """
        return self._host

    security.declarePublic('getExtension')
    def getExtension(self):
        """
        Return the extension for the server
        """
        return self._extension

    security.declarePublic('getUploadLogin')
    def getUploadLogin(self):
        """
        Return the ftpupload user for the server
        """
        return self._upload_login

    security.declarePublic('getUploadPassword')
    def getUploadPassword(self):
        """
        Return the upload password for the server
        """
        return self._upload_password

    security.declarePublic('getUploadDir')
    def getUploadDir(self):
        """
        Return the upload dir for the server
        """
        return self._upload_dir

    security.declarePublic('getCinemaSkin')
    def getCinemaSkin(self):
        """
        Return the cinema skin for the server
        """
        return self._def_cinemaskin

    security.declarePublic('getSyndicationSkin')
    def getSyndicationSkin(self):
        """
        Return the syndication skin for the server
        """
        return self._def_syndicationskin

    security.declarePublic('getWidth')
    def getWidth(self):
        """
        Return the default width for this server
        """
        return self._width

    security.declarePublic('getHeight')
    def getHeight(self):
        """
        Return the default height for this server
        """
        return self._height

    security.declarePublic('getMetaGenerator')
    def getMetaGenerator(self):
        """
        Return what type of meta generator the server should use
        """
        gen = self._meta_generator
        if gen == 'None':
            return None
        return gen

    security.declareProtected(CMFCorePermissions.View, 'getEditorPlayerSkin')
    def getEditorPlayerSkin(self):
        """
        Get the name of the template that contains the editor player skin
        """
        return self._editor_player_skin

    security.declareProtected(CMFCorePermissions.View, 'getCinemaPlayerSkin')
    def getCinemaPlayerSkin(self):
        """
        Get the name of the template that contains the editor player skin
        """
        return self._cinema_player_skin


    security.declareProtected(CMFCorePermissions.View, 'getPlayUrl')
    def getPlayUrl(self, filename):
        """
        Return a url for the requested filename
        """
        url = '%s://%s:%d%s%s%s' % (self._protocol, self._servername, self._port, self._contentDir, filename, self._extension)
        return url
        
    def Provides(self):
        """
        Return a tuple containing the mime-types the plugin provides
        """
        return self._provides
        
    def getSelectedFormat(self):
        """
        Return the selected format for the plugin
        """
        return self._sel_format

    def getDataListForServerInfo(self):
        """
        Return a list of dictionaries containing the info a ServerInfo need as default settings
        """

    def getDataListForServer(self):
        """
        Return a list of dictionaries containing the info a server needs to edit
        """
