##############################################################################
#    Copyright (C) 2002  MM Manager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import Interface

class IServerPlugin(Interface.Base):
    """
    General interface for MyMediaManager server plugins.

    These methods need to be implemented by any class that wants to register
    as a server plugin.
    """

    def getProtocol(self):
        """
        Return the protocol for the server.
        The protocol is a string containing e.g. 'mmx' or 'rtsp'.
        """

    def getHost(self):
        """
        Return the hostname or IP for the server.
        """

    def getExtension(self):
        """
        Return the extension for the server.
        This is the filename extension for the physical mediafiles
        e.g. 'rm' or 'asx'.
        """

    def getUploadLogin(self):
        """
        Return the upload user for the server.
        Upload login, UploadPassword and UploadDir is used for
        uploading mediafiles to the streaming server. The server plugin
        implements its own upload methods using e.g. ftp or webdav.
        """

    def getUploadPassword(self):
        """
        Return the upload password for the server
        """

    def getUploadDir(self):
        """
        Return the upload dir for the server
        """

    def getCinemaSkin(self):
        """
        Return the cinema skin for the server.
        The cinema skin is a macro from liveevent_view_macros that
        is used for displaying the event.
        """

    def getSyndicationSkin(self):
        """
        Return the syndication skin for the server.
        The syndication skin is used when e.g. sending out mails
        with links to events.
        """

    def getWidth(self):
        """
        Return the default width for this server
        """

    def getHeight(self):
        """
        Return the default height for this server
        """

    def getMetaGenerator(self):
        """
        Return what type of meta generator the server should use.
        """

    def getEditorPlayerSkin(self):
        """
        Get the name of the template that contains the editor player skin
        """

    def getCinemaPlayerSkin(self):
        """
        Get the name of the template that contains the editor player skin
        """

    def getPlayUrl(self, filename):
        """
        Return a url for the requested filename
        """

    def Provides(self):
        """
        Return a tuple containing the mime-types the plugin provides
        """

    def getSelectedFormat(self):
        """
        Return the selected format for the plugin.
        The format is the streaming format used for a specific
        instance of this plugin e.g. 'video/x-ms-asf'.
        """
        
    def getDataListForServerInfo(self):
        """
        Return a list of dictionaries containing the info a ServerInfo need as default settings
        """
